classdef GradienteBLI < Base
    %GrandienteBLI 
    %   Calcula el minimo de una funcion con el metodo del gradiente
    %   y para el calculo del paso utiliza un backtraking con condicion
    %   de Armijo.
    %   Esta clase debe ser extendida por otra clase que implemente la 
    %   funcion que se quiere minimizar
    
    properties        
    end

    methods       
        function obj = GradienteBLI(arg)
            obj = obj@Base(arg);
            if ~isempty(arg)
                obj.eta = arg(1);
                obj.ro = arg(2);
            end
        end    
        function [d_x,d_y] = d(obj,arg)
            [fd_x,fd_y] = obj.grad_f(arg);
            d_x= -1 * fd_x;
            d_y= -1 * fd_y;
        end          
        function [ret]=metodogradiente(obj,xk)
            
            fid = fopen(strcat('stats',obj.statfilename,'.txt'),'w');
            fprintf(fid,'k\tkx\t||xk-xmin||\t||grad_f(xk)||\n');

            fid_tex = fopen(strcat('stats',obj.statfilename,'.tex'),'w');
            fprintf(fid_tex,'\\underline{Funcion:} $%s$ \\\\ \nParametros: $\\eta = %f; \\rho = %f; x_0 = (%f;%f)$; $x_*=(%f,%f)$ \\\\ \n',obj.statfilename,obj.eta,obj.ro,xk(1),xk(2),obj.xmin(1),obj.xmin(2));
            fprintf(fid_tex,'\\begin{tabular}{|c|c|c|c|}\\hline\n\t$k$&$x_k$&$||x_k-x_*||$&$||grad_f(x_k)||$\\\\ \\hline');

            parar = false;
            k = 0;
            while ~parar
                [g1,g2] = obj.grad_f(xk);
                normagrad = obj.norma([g1;g2]);         
                fprintf(fid,'%d\t(%f,%f)\t%f\t%f\n',k,xk(1),xk(2),obj.norma(xk-obj.xmin),normagrad);
                fprintf(fid_tex,'\n\t%d&(%f,%f)&%f&%f\\\\ \\hline',k,xk(1),xk(2),obj.norma(xk-obj.xmin),normagrad);
                if normagrad < 10^-5 %condicion de terminacion del algoritmos
                    parar = true;
                else
                    [d1,d2] = obj.d(xk);
                    d = [d1;d2];
                    lambda = obj.backtracking(xk);
                    xk = xk+lambda*d;
                end
                k = k+1;
                if(k>1000) % Seguro contra estancamiento del algoritmo
                    parar=true;
                end
            end
            fprintf(fid_tex,'\n\\end{tabular}\\\\\\\\');
            
            fclose(fid_tex);
            fclose(fid);

            ret = xk;
        end
    end
end

